/*-----------------------------------------------*\

Author: Thomas Geisen (www.thomasgeisen.fr)
Method : SUITCSS (modified)

--- This file is only for RTL languages

\*-----------------------------------------------*/
.wpr-wrap {
  margin: 0 -20px 0 0;
}

#hs-beacon iframe:nth-child(1) {
  right: inherit !important;
  left: 18px !important;
}
#hs-beacon iframe:nth-child(2) {
  right: inherit !important;
  left: 6px !important;
}

.wpr-Page-col--fixed {
  margin-left: 0;
  margin-right: 24px;
}
@media (max-width: 1239px) {
  .wpr-Page-col--fixed {
    margin-right: 0;
  }
}

.wpr-Content {
  border: 1px solid #E0E4E9;
  border-right: none;
}

.wpr-Content-tips {
  right: unset;
  left: 24px;
}

.wpr-Sidebar-notice {
  border: 1px solid #E8EBEE;
  border-right: 2px solid #1EADBF;
}
.wpr-Sidebar .wpr-Sidebar-info h4 {
  padding-left: inherit;
  padding-right: 56px;
}

.wpr-menuItem {
  padding: 16px 20px 18px 44px;
}
.wpr-menuItem:before {
  right: inherit !important;
  left: 18px !important;
}
.wpr-menuItem:after {
  border-width: 12px 0 12px 10px;
  border-color: transparent transparent transparent #fff;
  left: 0;
  right: inherit;
  transform: translateX(-12px);
}
.wpr-menuItem:hover {
  transform: translateX(0);
}
.wpr-menuItem.wpr-subMenuItem {
  padding: 10px 25px 8px 20px;
}

.wpr-sectionHeader:before {
  left: inherit;
  right: 0;
}
.wpr-sectionHeader .wpr-title1:before {
  margin-left: 24px;
  margin-right: 0;
}

.wpr-optionHeader .wpr-title2 {
  padding-right: 0;
  padding-left: 40px;
}

.wpr-infoAction:before {
  right: -26px;
}

.wpr-fieldWarning {
  padding: 16px 56px 24px 16px;
}
.wpr-fieldWarning:after {
  left: inherit;
  right: 20px;
}
.wpr-fieldWarning:before {
  left: inherit;
  right: -16px;
}
.wpr-fieldWarning-title:before {
  left: inherit;
  right: -40px;
}
.wpr-fieldWarning.wpr-radio-warning {
  margin-left: 0;
  margin-right: -32px;
}
.wpr-fieldWarning.wpr-radio-warning:after {
  left: inherit;
  right: 40px;
}

@media only screen and (max-width: 400px) {
  .wpr-fieldWarning.wpr-radio-warning {
    margin-left: 0;
    margin-right: -32px;
  }
  .wpr-fieldWarning.wpr-radio-warning .wpr-button {
    padding-right: 8px;
    padding-left: 32px;
  }
}
@media only screen and (max-width: 350px) {
  .wpr-radio-buttons {
    padding-left: 0;
    padding-right: 24px;
  }

  .wpr-fieldWarning.wpr-radio-warning {
    margin-left: 0;
    margin-right: -24px;
    padding-left: 0;
    padding-right: 30px;
  }
  .wpr-fieldWarning.wpr-radio-warning .wpr-button {
    white-space: normal;
    padding-right: 8px;
    padding-left: 25px;
  }
}
.wpr-checkbox {
  padding-left: 0;
  padding-right: 32px;
}
.wpr-checkbox [type=checkbox]:not(:checked),
.wpr-checkbox [type=checkbox]:checked {
  right: -9999px;
}
.wpr-checkbox [type=checkbox]:not(:checked) + label:before,
.wpr-checkbox [type=checkbox]:checked + label:before {
  left: inherit;
  right: 0;
}
.wpr-checkbox [type=checkbox]:not(:checked) + label:after,
.wpr-checkbox [type=checkbox]:checked + label:after {
  left: inherit;
  right: 2px;
}

.wpr-radio {
  padding-left: 0;
  padding-right: 88px;
}
.wpr-radio [type=checkbox]:not(:checked),
.wpr-radio [type=checkbox]:checked {
  right: -9999px;
}
.wpr-radio [type=checkbox]:not(:checked) + label:before,
.wpr-radio [type=checkbox]:checked + label:before {
  left: inherit;
  right: 0;
}
.wpr-radio [type=checkbox]:not(:checked) + label:after,
.wpr-radio [type=checkbox]:checked + label:after {
  left: inherit;
  right: 3px;
}
.wpr-radio [type=checkbox]:checked + label:after {
  right: 33px;
}
.wpr-radio [type=checkbox]:checked + label .wpr-radio-ui,
.wpr-radio [type=checkbox]:not(:checked) + label .wpr-radio-ui:before,
.wpr-radio [type=checkbox]:checked + label .wpr-radio-ui:after {
  right: 4px;
}
.wpr-radio [type=checkbox]:not(:checked) + label .wpr-radio-ui:before {
  right: 27px;
}
.wpr-radio--reverse {
  padding-left: 0;
  padding-right: 72px;
}
.wpr-radio--reverse [type=checkbox]:not(:checked) + label:before,
.wpr-radio--reverse [type=checkbox]:checked + label:before {
  right: 0;
  left: inherit;
}
.wpr-radio--reverse [type=checkbox]:not(:checked) + label:after,
.wpr-radio--reverse [type=checkbox]:checked + label:after {
  right: 33px;
  left: inherit;
}
.wpr-radio--reverse [type=checkbox]:checked + label:after {
  right: 3px;
  left: inherit;
}
.wpr-radio--reverse [type=checkbox]:checked + label .wpr-radio-ui,
.wpr-radio--reverse [type=checkbox]:not(:checked) + label .wpr-radio-ui:before,
.wpr-radio--reverse [type=checkbox]:checked + label .wpr-radio-ui:after {
  right: 15px;
  left: inherit;
}
.wpr-radio--reverse [type=checkbox]:not(:checked) + label .wpr-radio-ui:before {
  right: 6px;
  left: inherit;
}

.wpr-multiple .wpr-button {
  margin-right: 16px;
  margin-left: 0;
}
.wpr-multiple-close {
  margin-right: 0;
  margin-left: 16px;
}

.wpr-addon .wpr-flex > div {
  text-align: right;
}
.wpr-addon .wpr-addon-text {
  margin-left: inherit;
  margin-right: 32px;
}
@media (max-width: 1239px) {
  .wpr-addon .wpr-addon-text {
    margin-right: 16px;
  }
}
@media (max-width: 1083px) {
  .wpr-addon .wpr-addon-text {
    margin-right: 32px;
  }
}
@media (max-width: 783px) {
  .wpr-addon .wpr-addon-text {
    margin-right: 0;
  }
}

.wpr-tools-col:first-child {
  padding-right: 72px;
  padding-left: 24px;
}
.wpr-tools-col:last-child {
  text-align: left;
}
@media (max-width: 783px) {
  .wpr-tools-col:last-child {
    text-align: right;
  }
}
.wpr-tools-label:before {
  left: inherit;
  right: 0;
}

.wpr-field .wpr-flex--egal > div:last-child {
  text-align: left;
}
.wpr-field-list li:before {
  display: inline-block;
  margin-right: 0;
  margin-left: 8px;
}
.wpr-field--split {
  padding-right: 0;
}
.wpr-field--split + .wpr-field--split {
  padding-left: 0;
  padding-right: 16px;
}
.wpr-field--children {
  padding-left: 0;
  padding-right: 32px;
}
.wpr-field--children.wpr-field--textarea {
  padding-left: 80px;
  padding-right: 32px;
}
@media (max-width: 1239px) {
  .wpr-field--children.wpr-field--textarea {
    padding-left: 32px;
    padding-right: 0;
  }
}
@media (max-width: 783px) {
  .wpr-field--children.wpr-field--textarea {
    padding-left: 0;
  }
}
.wpr-field--children.no-space {
  padding-right: 0;
}
.wpr-field--checkbox .wpr-field-description {
  margin-left: 0;
  margin-right: 32px;
}
.wpr-field--radio .wpr-field-description {
  margin-left: 0;
  margin-right: 88px;
}

.wpr-adblock img {
  margin-right: 0;
  margin-left: 16px;
}
.wpr-adblock-close {
  right: inherit;
  left: 24px;
}

.wpr-notice {
  background-position: 10% bottom;
}
.wpr-notice-container {
  padding: 24px 40px 24px 25%;
}
.wpr-notice-close {
  right: inherit;
  left: 24px;
}