<?php

// autoload_psr4.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'WP_Rocket\\' => array($baseDir . '/inc'),
    'WPMedia\\Cloudflare\\' => array($baseDir . '/inc/Addon/Cloudflare'),
    'Psr\\Log\\' => array($vendorDir . '/psr/log/Psr/Log'),
    'Monolog\\' => array($vendorDir . '/monolog/monolog/src/Monolog'),
);
