<?php
/**
 * Basic exception
 *
 * Please report bugs on https://github.com/matthiasmullie/minify/issues
 *
 * @author Matthias Mullie <minify@mullie.eu>
 * @copyright Copyright (c) 2012, Matthias Mullie. All rights reserved
 * @license MIT License
 */
namespace WP_Rocket\Dependencies\Minify\Exceptions;

use WP_Rocket\Dependencies\Minify\Exception;

/**
 * Basic Exception Class
 *
 * @package Minify\Exception
 * @author Matthias Mullie <minify@mullie.eu>
 */
abstract class BasicException extends Exception
{
}
