<?php
/**
 * Upgrade section template.
 *
 * @since 3.7.3
 */

defined( 'ABSPATH' ) || exit;
?>
<p>
<?php esc_html_e( 'You can use WP Rocket on more websites by upgrading your license (you will only pay the price difference between your current and new licenses).', 'rocket' ); ?> <button class="wpr-license-upgrade-button wpr-popin-upgrade-toggle"><?php esc_html_e( 'Upgrade now', 'rocket' ); ?></button>
</p>
