<?php

// autoload_static.php @generated by Composer

namespace Composer\Autoload;

class ComposerStaticInita9a3a96e670bf58ac4dde9996f742c3d
{
    public static $prefixLengthsPsr4 = array (
        'W' => 
        array (
            'WP_Rocket\\' => 10,
            'WPMedia\\Cloudflare\\' => 19,
        ),
        'P' => 
        array (
            'Psr\\Log\\' => 8,
        ),
        'M' => 
        array (
            'Monolog\\' => 8,
        ),
    );

    public static $prefixDirsPsr4 = array (
        'WP_Rocket\\' => 
        array (
            0 => __DIR__ . '/../..' . '/inc',
        ),
        'WPMedia\\Cloudflare\\' => 
        array (
            0 => __DIR__ . '/../..' . '/inc/Addon/Cloudflare',
        ),
        'Psr\\Log\\' => 
        array (
            0 => __DIR__ . '/..' . '/psr/log/Psr/Log',
        ),
        'Monolog\\' => 
        array (
            0 => __DIR__ . '/..' . '/monolog/monolog/src/Monolog',
        ),
    );

    public static $classMap = array (
        'Composer\\InstalledVersions' => __DIR__ . '/..' . '/composer/InstalledVersions.php',
        'Imagify_Partner' => __DIR__ . '/../..' . '/inc/vendors/classes/class-imagify-partner.php',
        'Minify_CSS_UriRewriter' => __DIR__ . '/../..' . '/inc/vendors/classes/class-minify-css-urirewriter.php',
        'Minify_HTML' => __DIR__ . '/../..' . '/inc/deprecated/vendors/classes/class-minify-html.php',
        'Monolog\\Attribute\\AsMonologProcessor' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Attribute/AsMonologProcessor.php',
        'Monolog\\DateTimeImmutable' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/DateTimeImmutable.php',
        'Monolog\\ErrorHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/ErrorHandler.php',
        'Monolog\\Formatter\\ChromePHPFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/ChromePHPFormatter.php',
        'Monolog\\Formatter\\ElasticaFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/ElasticaFormatter.php',
        'Monolog\\Formatter\\ElasticsearchFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/ElasticsearchFormatter.php',
        'Monolog\\Formatter\\FlowdockFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/FlowdockFormatter.php',
        'Monolog\\Formatter\\FluentdFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/FluentdFormatter.php',
        'Monolog\\Formatter\\FormatterInterface' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/FormatterInterface.php',
        'Monolog\\Formatter\\GelfMessageFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/GelfMessageFormatter.php',
        'Monolog\\Formatter\\GoogleCloudLoggingFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/GoogleCloudLoggingFormatter.php',
        'Monolog\\Formatter\\HtmlFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/HtmlFormatter.php',
        'Monolog\\Formatter\\JsonFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/JsonFormatter.php',
        'Monolog\\Formatter\\LineFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/LineFormatter.php',
        'Monolog\\Formatter\\LogglyFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/LogglyFormatter.php',
        'Monolog\\Formatter\\LogmaticFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/LogmaticFormatter.php',
        'Monolog\\Formatter\\LogstashFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/LogstashFormatter.php',
        'Monolog\\Formatter\\MongoDBFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/MongoDBFormatter.php',
        'Monolog\\Formatter\\NormalizerFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/NormalizerFormatter.php',
        'Monolog\\Formatter\\ScalarFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/ScalarFormatter.php',
        'Monolog\\Formatter\\WildfireFormatter' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Formatter/WildfireFormatter.php',
        'Monolog\\Handler\\AbstractHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/AbstractHandler.php',
        'Monolog\\Handler\\AbstractProcessingHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/AbstractProcessingHandler.php',
        'Monolog\\Handler\\AbstractSyslogHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/AbstractSyslogHandler.php',
        'Monolog\\Handler\\AmqpHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/AmqpHandler.php',
        'Monolog\\Handler\\BrowserConsoleHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/BrowserConsoleHandler.php',
        'Monolog\\Handler\\BufferHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/BufferHandler.php',
        'Monolog\\Handler\\ChromePHPHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/ChromePHPHandler.php',
        'Monolog\\Handler\\CouchDBHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/CouchDBHandler.php',
        'Monolog\\Handler\\CubeHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/CubeHandler.php',
        'Monolog\\Handler\\Curl\\Util' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/Curl/Util.php',
        'Monolog\\Handler\\DeduplicationHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/DeduplicationHandler.php',
        'Monolog\\Handler\\DoctrineCouchDBHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/DoctrineCouchDBHandler.php',
        'Monolog\\Handler\\DynamoDbHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/DynamoDbHandler.php',
        'Monolog\\Handler\\ElasticaHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/ElasticaHandler.php',
        'Monolog\\Handler\\ElasticsearchHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/ElasticsearchHandler.php',
        'Monolog\\Handler\\ErrorLogHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/ErrorLogHandler.php',
        'Monolog\\Handler\\FallbackGroupHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/FallbackGroupHandler.php',
        'Monolog\\Handler\\FilterHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/FilterHandler.php',
        'Monolog\\Handler\\FingersCrossedHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/FingersCrossedHandler.php',
        'Monolog\\Handler\\FingersCrossed\\ActivationStrategyInterface' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/FingersCrossed/ActivationStrategyInterface.php',
        'Monolog\\Handler\\FingersCrossed\\ChannelLevelActivationStrategy' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/FingersCrossed/ChannelLevelActivationStrategy.php',
        'Monolog\\Handler\\FingersCrossed\\ErrorLevelActivationStrategy' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/FingersCrossed/ErrorLevelActivationStrategy.php',
        'Monolog\\Handler\\FirePHPHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/FirePHPHandler.php',
        'Monolog\\Handler\\FleepHookHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/FleepHookHandler.php',
        'Monolog\\Handler\\FlowdockHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/FlowdockHandler.php',
        'Monolog\\Handler\\FormattableHandlerInterface' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/FormattableHandlerInterface.php',
        'Monolog\\Handler\\FormattableHandlerTrait' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/FormattableHandlerTrait.php',
        'Monolog\\Handler\\GelfHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/GelfHandler.php',
        'Monolog\\Handler\\GroupHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/GroupHandler.php',
        'Monolog\\Handler\\Handler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/Handler.php',
        'Monolog\\Handler\\HandlerInterface' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/HandlerInterface.php',
        'Monolog\\Handler\\HandlerWrapper' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/HandlerWrapper.php',
        'Monolog\\Handler\\IFTTTHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/IFTTTHandler.php',
        'Monolog\\Handler\\InsightOpsHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/InsightOpsHandler.php',
        'Monolog\\Handler\\LogEntriesHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/LogEntriesHandler.php',
        'Monolog\\Handler\\LogglyHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/LogglyHandler.php',
        'Monolog\\Handler\\LogmaticHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/LogmaticHandler.php',
        'Monolog\\Handler\\MailHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/MailHandler.php',
        'Monolog\\Handler\\MandrillHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/MandrillHandler.php',
        'Monolog\\Handler\\MissingExtensionException' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/MissingExtensionException.php',
        'Monolog\\Handler\\MongoDBHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/MongoDBHandler.php',
        'Monolog\\Handler\\NativeMailerHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/NativeMailerHandler.php',
        'Monolog\\Handler\\NewRelicHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/NewRelicHandler.php',
        'Monolog\\Handler\\NoopHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/NoopHandler.php',
        'Monolog\\Handler\\NullHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/NullHandler.php',
        'Monolog\\Handler\\OverflowHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/OverflowHandler.php',
        'Monolog\\Handler\\PHPConsoleHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/PHPConsoleHandler.php',
        'Monolog\\Handler\\ProcessHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/ProcessHandler.php',
        'Monolog\\Handler\\ProcessableHandlerInterface' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/ProcessableHandlerInterface.php',
        'Monolog\\Handler\\ProcessableHandlerTrait' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/ProcessableHandlerTrait.php',
        'Monolog\\Handler\\PsrHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/PsrHandler.php',
        'Monolog\\Handler\\PushoverHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/PushoverHandler.php',
        'Monolog\\Handler\\RedisHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/RedisHandler.php',
        'Monolog\\Handler\\RedisPubSubHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/RedisPubSubHandler.php',
        'Monolog\\Handler\\RollbarHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/RollbarHandler.php',
        'Monolog\\Handler\\RotatingFileHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/RotatingFileHandler.php',
        'Monolog\\Handler\\SamplingHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/SamplingHandler.php',
        'Monolog\\Handler\\SendGridHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/SendGridHandler.php',
        'Monolog\\Handler\\SlackHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/SlackHandler.php',
        'Monolog\\Handler\\SlackWebhookHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/SlackWebhookHandler.php',
        'Monolog\\Handler\\Slack\\SlackRecord' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/Slack/SlackRecord.php',
        'Monolog\\Handler\\SocketHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/SocketHandler.php',
        'Monolog\\Handler\\SqsHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/SqsHandler.php',
        'Monolog\\Handler\\StreamHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/StreamHandler.php',
        'Monolog\\Handler\\SwiftMailerHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/SwiftMailerHandler.php',
        'Monolog\\Handler\\SymfonyMailerHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/SymfonyMailerHandler.php',
        'Monolog\\Handler\\SyslogHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/SyslogHandler.php',
        'Monolog\\Handler\\SyslogUdpHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/SyslogUdpHandler.php',
        'Monolog\\Handler\\SyslogUdp\\UdpSocket' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/SyslogUdp/UdpSocket.php',
        'Monolog\\Handler\\TelegramBotHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/TelegramBotHandler.php',
        'Monolog\\Handler\\TestHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/TestHandler.php',
        'Monolog\\Handler\\WebRequestRecognizerTrait' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/WebRequestRecognizerTrait.php',
        'Monolog\\Handler\\WhatFailureGroupHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/WhatFailureGroupHandler.php',
        'Monolog\\Handler\\ZendMonitorHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Handler/ZendMonitorHandler.php',
        'Monolog\\LogRecord' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/LogRecord.php',
        'Monolog\\Logger' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Logger.php',
        'Monolog\\Processor\\GitProcessor' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Processor/GitProcessor.php',
        'Monolog\\Processor\\HostnameProcessor' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Processor/HostnameProcessor.php',
        'Monolog\\Processor\\IntrospectionProcessor' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Processor/IntrospectionProcessor.php',
        'Monolog\\Processor\\MemoryPeakUsageProcessor' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Processor/MemoryPeakUsageProcessor.php',
        'Monolog\\Processor\\MemoryProcessor' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Processor/MemoryProcessor.php',
        'Monolog\\Processor\\MemoryUsageProcessor' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Processor/MemoryUsageProcessor.php',
        'Monolog\\Processor\\MercurialProcessor' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Processor/MercurialProcessor.php',
        'Monolog\\Processor\\ProcessIdProcessor' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Processor/ProcessIdProcessor.php',
        'Monolog\\Processor\\ProcessorInterface' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Processor/ProcessorInterface.php',
        'Monolog\\Processor\\PsrLogMessageProcessor' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Processor/PsrLogMessageProcessor.php',
        'Monolog\\Processor\\TagProcessor' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Processor/TagProcessor.php',
        'Monolog\\Processor\\UidProcessor' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Processor/UidProcessor.php',
        'Monolog\\Processor\\WebProcessor' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Processor/WebProcessor.php',
        'Monolog\\Registry' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Registry.php',
        'Monolog\\ResettableInterface' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/ResettableInterface.php',
        'Monolog\\SignalHandler' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/SignalHandler.php',
        'Monolog\\Test\\TestCase' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Test/TestCase.php',
        'Monolog\\Utils' => __DIR__ . '/..' . '/monolog/monolog/src/Monolog/Utils.php',
        'Psr\\Log\\AbstractLogger' => __DIR__ . '/..' . '/psr/log/Psr/Log/AbstractLogger.php',
        'Psr\\Log\\InvalidArgumentException' => __DIR__ . '/..' . '/psr/log/Psr/Log/InvalidArgumentException.php',
        'Psr\\Log\\LogLevel' => __DIR__ . '/..' . '/psr/log/Psr/Log/LogLevel.php',
        'Psr\\Log\\LoggerAwareInterface' => __DIR__ . '/..' . '/psr/log/Psr/Log/LoggerAwareInterface.php',
        'Psr\\Log\\LoggerAwareTrait' => __DIR__ . '/..' . '/psr/log/Psr/Log/LoggerAwareTrait.php',
        'Psr\\Log\\LoggerInterface' => __DIR__ . '/..' . '/psr/log/Psr/Log/LoggerInterface.php',
        'Psr\\Log\\LoggerTrait' => __DIR__ . '/..' . '/psr/log/Psr/Log/LoggerTrait.php',
        'Psr\\Log\\NullLogger' => __DIR__ . '/..' . '/psr/log/Psr/Log/NullLogger.php',
        'Psr\\Log\\Test\\DummyTest' => __DIR__ . '/..' . '/psr/log/Psr/Log/Test/DummyTest.php',
        'Psr\\Log\\Test\\LoggerInterfaceTest' => __DIR__ . '/..' . '/psr/log/Psr/Log/Test/LoggerInterfaceTest.php',
        'Psr\\Log\\Test\\TestLogger' => __DIR__ . '/..' . '/psr/log/Psr/Log/Test/TestLogger.php',
        'WPMedia\\Cloudflare\\APIClient' => __DIR__ . '/../..' . '/inc/Addon/Cloudflare/APIClient.php',
        'WPMedia\\Cloudflare\\AuthenticationException' => __DIR__ . '/../..' . '/inc/Addon/Cloudflare/AuthenticationException.php',
        'WPMedia\\Cloudflare\\Cloudflare' => __DIR__ . '/../..' . '/inc/Addon/Cloudflare/Cloudflare.php',
        'WPMedia\\Cloudflare\\Subscriber' => __DIR__ . '/../..' . '/inc/Addon/Cloudflare/Subscriber.php',
        'WPMedia\\Cloudflare\\UnauthorizedException' => __DIR__ . '/../..' . '/inc/Addon/Cloudflare/UnauthorizedException.php',
        'WP_Rocket\\Abstract_Render' => __DIR__ . '/../..' . '/inc/classes/class-abstract-render.php',
        'WP_Rocket\\Addon\\Busting\\BustingFactory' => __DIR__ . '/../..' . '/inc/deprecated/Engine/Addon/Busting/BustingFactory.php',
        'WP_Rocket\\Addon\\Busting\\FileBustingTrait' => __DIR__ . '/../..' . '/inc/deprecated/Engine/Addon/Busting/FileBustingTrait.php',
        'WP_Rocket\\Addon\\FacebookTracking\\Subscriber' => __DIR__ . '/../..' . '/inc/deprecated/Engine/Addon/FacebookTracking/Subscriber.php',
        'WP_Rocket\\Addon\\GoogleTracking\\GoogleAnalytics' => __DIR__ . '/../..' . '/inc/deprecated/Engine/Addon/GoogleTracking/GoogleAnalytics.php',
        'WP_Rocket\\Addon\\GoogleTracking\\GoogleTagManager' => __DIR__ . '/../..' . '/inc/deprecated/Engine/Addon/GoogleTracking/GoogleTagManager.php',
        'WP_Rocket\\Addon\\GoogleTracking\\Subscriber' => __DIR__ . '/../..' . '/inc/deprecated/Engine/Addon/GoogleTracking/Subscriber.php',
        'WP_Rocket\\Addon\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Addon/ServiceProvider.php',
        'WP_Rocket\\Addon\\Sucuri\\Subscriber' => __DIR__ . '/../..' . '/inc/Addon/Sucuri/Subscriber.php',
        'WP_Rocket\\Addon\\Varnish\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Addon/Varnish/ServiceProvider.php',
        'WP_Rocket\\Addon\\Varnish\\Subscriber' => __DIR__ . '/../..' . '/inc/Addon/Varnish/Subscriber.php',
        'WP_Rocket\\Addon\\Varnish\\Varnish' => __DIR__ . '/../..' . '/inc/Addon/Varnish/Varnish.php',
        'WP_Rocket\\Admin\\Abstract_Options' => __DIR__ . '/../..' . '/inc/classes/admin/class-abstract-options.php',
        'WP_Rocket\\Admin\\Logs' => __DIR__ . '/../..' . '/inc/classes/admin/class-logs.php',
        'WP_Rocket\\Admin\\Options' => __DIR__ . '/../..' . '/inc/classes/admin/class-options.php',
        'WP_Rocket\\Admin\\Options_Data' => __DIR__ . '/../..' . '/inc/classes/admin/class-options-data.php',
        'WP_Rocket\\Buffer\\Abstract_Buffer' => __DIR__ . '/../..' . '/inc/classes/Buffer/class-abstract-buffer.php',
        'WP_Rocket\\Buffer\\Cache' => __DIR__ . '/../..' . '/inc/classes/Buffer/class-cache.php',
        'WP_Rocket\\Buffer\\Config' => __DIR__ . '/../..' . '/inc/classes/Buffer/class-config.php',
        'WP_Rocket\\Buffer\\Tests' => __DIR__ . '/../..' . '/inc/classes/Buffer/class-tests.php',
        'WP_Rocket\\Busting\\Abstract_Busting' => __DIR__ . '/../..' . '/inc/deprecated/classes/busting/class-abstract-busting.php',
        'WP_Rocket\\Busting\\Facebook_Pickles' => __DIR__ . '/../..' . '/inc/deprecated/classes/busting/class-facebook-pickles.php',
        'WP_Rocket\\Busting\\Facebook_SDK' => __DIR__ . '/../..' . '/inc/deprecated/classes/busting/class-facebook-sdk.php',
        'WP_Rocket\\Dependencies\\Database\\Base' => __DIR__ . '/../..' . '/inc/Dependencies/Database/Base.php',
        'WP_Rocket\\Dependencies\\Database\\Column' => __DIR__ . '/../..' . '/inc/Dependencies/Database/Column.php',
        'WP_Rocket\\Dependencies\\Database\\Queries\\Compare' => __DIR__ . '/../..' . '/inc/Dependencies/Database/Queries/Compare.php',
        'WP_Rocket\\Dependencies\\Database\\Queries\\Date' => __DIR__ . '/../..' . '/inc/Dependencies/Database/Queries/Date.php',
        'WP_Rocket\\Dependencies\\Database\\Queries\\Meta' => __DIR__ . '/../..' . '/inc/Dependencies/Database/Queries/Meta.php',
        'WP_Rocket\\Dependencies\\Database\\Query' => __DIR__ . '/../..' . '/inc/Dependencies/Database/Query.php',
        'WP_Rocket\\Dependencies\\Database\\Row' => __DIR__ . '/../..' . '/inc/Dependencies/Database/Row.php',
        'WP_Rocket\\Dependencies\\Database\\Schema' => __DIR__ . '/../..' . '/inc/Dependencies/Database/Schema.php',
        'WP_Rocket\\Dependencies\\Database\\Table' => __DIR__ . '/../..' . '/inc/Dependencies/Database/Table.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Argument\\ArgumentResolverInterface' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Argument/ArgumentResolverInterface.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Argument\\ArgumentResolverTrait' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Argument/ArgumentResolverTrait.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Argument\\ClassName' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Argument/ClassName.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Argument\\ClassNameInterface' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Argument/ClassNameInterface.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Argument\\ClassNameWithOptionalValue' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Argument/ClassNameWithOptionalValue.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Argument\\RawArgument' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Argument/RawArgument.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Argument\\RawArgumentInterface' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Argument/RawArgumentInterface.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Container' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Container.php',
        'WP_Rocket\\Dependencies\\League\\Container\\ContainerAwareInterface' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/ContainerAwareInterface.php',
        'WP_Rocket\\Dependencies\\League\\Container\\ContainerAwareTrait' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/ContainerAwareTrait.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Definition\\Definition' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Definition/Definition.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Definition\\DefinitionAggregate' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Definition/DefinitionAggregate.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Definition\\DefinitionAggregateInterface' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Definition/DefinitionAggregateInterface.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Definition\\DefinitionInterface' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Definition/DefinitionInterface.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Exception\\ContainerException' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Exception/ContainerException.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Exception\\NotFoundException' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Exception/NotFoundException.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Inflector\\Inflector' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Inflector/Inflector.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Inflector\\InflectorAggregate' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Inflector/InflectorAggregate.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Inflector\\InflectorAggregateInterface' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Inflector/InflectorAggregateInterface.php',
        'WP_Rocket\\Dependencies\\League\\Container\\Inflector\\InflectorInterface' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/Inflector/InflectorInterface.php',
        'WP_Rocket\\Dependencies\\League\\Container\\ReflectionContainer' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/ReflectionContainer.php',
        'WP_Rocket\\Dependencies\\League\\Container\\ServiceProvider\\AbstractServiceProvider' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/ServiceProvider/AbstractServiceProvider.php',
        'WP_Rocket\\Dependencies\\League\\Container\\ServiceProvider\\BootableServiceProviderInterface' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/ServiceProvider/BootableServiceProviderInterface.php',
        'WP_Rocket\\Dependencies\\League\\Container\\ServiceProvider\\ServiceProviderAggregate' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/ServiceProvider/ServiceProviderAggregate.php',
        'WP_Rocket\\Dependencies\\League\\Container\\ServiceProvider\\ServiceProviderAggregateInterface' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/ServiceProvider/ServiceProviderAggregateInterface.php',
        'WP_Rocket\\Dependencies\\League\\Container\\ServiceProvider\\ServiceProviderInterface' => __DIR__ . '/../..' . '/inc/Dependencies/League/Container/ServiceProvider/ServiceProviderInterface.php',
        'WP_Rocket\\Dependencies\\Minify\\CSS' => __DIR__ . '/../..' . '/inc/Dependencies/Minify/CSS.php',
        'WP_Rocket\\Dependencies\\Minify\\Exception' => __DIR__ . '/../..' . '/inc/Dependencies/Minify/Exception.php',
        'WP_Rocket\\Dependencies\\Minify\\Exceptions\\BasicException' => __DIR__ . '/../..' . '/inc/Dependencies/Minify/Exceptions/BasicException.php',
        'WP_Rocket\\Dependencies\\Minify\\Exceptions\\FileImportException' => __DIR__ . '/../..' . '/inc/Dependencies/Minify/Exceptions/FileImportException.php',
        'WP_Rocket\\Dependencies\\Minify\\Exceptions\\IOException' => __DIR__ . '/../..' . '/inc/Dependencies/Minify/Exceptions/IOException.php',
        'WP_Rocket\\Dependencies\\Minify\\JS' => __DIR__ . '/../..' . '/inc/Dependencies/Minify/JS.php',
        'WP_Rocket\\Dependencies\\Minify\\Minify' => __DIR__ . '/../..' . '/inc/Dependencies/Minify/Minify.php',
        'WP_Rocket\\Dependencies\\PathConverter\\Converter' => __DIR__ . '/../..' . '/inc/Dependencies/PathConverter/Converter.php',
        'WP_Rocket\\Dependencies\\PathConverter\\ConverterInterface' => __DIR__ . '/../..' . '/inc/Dependencies/PathConverter/ConverterInterface.php',
        'WP_Rocket\\Dependencies\\PathConverter\\NoConverter' => __DIR__ . '/../..' . '/inc/Dependencies/PathConverter/NoConverter.php',
        'WP_Rocket\\Dependencies\\Psr\\Container\\ContainerExceptionInterface' => __DIR__ . '/../..' . '/inc/Dependencies/Psr/Container/ContainerExceptionInterface.php',
        'WP_Rocket\\Dependencies\\Psr\\Container\\ContainerInterface' => __DIR__ . '/../..' . '/inc/Dependencies/Psr/Container/ContainerInterface.php',
        'WP_Rocket\\Dependencies\\Psr\\Container\\NotFoundExceptionInterface' => __DIR__ . '/../..' . '/inc/Dependencies/Psr/Container/NotFoundExceptionInterface.php',
        'WP_Rocket\\Dependencies\\RocketLazyload\\Assets' => __DIR__ . '/../..' . '/inc/Dependencies/RocketLazyload/Assets.php',
        'WP_Rocket\\Dependencies\\RocketLazyload\\Iframe' => __DIR__ . '/../..' . '/inc/Dependencies/RocketLazyload/Iframe.php',
        'WP_Rocket\\Dependencies\\RocketLazyload\\Image' => __DIR__ . '/../..' . '/inc/Dependencies/RocketLazyload/Image.php',
        'WP_Rocket\\Engine\\Activation\\Activation' => __DIR__ . '/../..' . '/inc/Engine/Activation/Activation.php',
        'WP_Rocket\\Engine\\Activation\\ActivationInterface' => __DIR__ . '/../..' . '/inc/Engine/Activation/ActivationInterface.php',
        'WP_Rocket\\Engine\\Activation\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Activation/ServiceProvider.php',
        'WP_Rocket\\Engine\\Admin\\API\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Admin/API/ServiceProvider.php',
        'WP_Rocket\\Engine\\Admin\\API\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Admin/API/Subscriber.php',
        'WP_Rocket\\Engine\\Admin\\ActionSchedulerSubscriber' => __DIR__ . '/../..' . '/inc/Engine/Admin/ActionSchedulerSubscriber.php',
        'WP_Rocket\\Engine\\Admin\\Beacon\\Beacon' => __DIR__ . '/../..' . '/inc/Engine/Admin/Beacon/Beacon.php',
        'WP_Rocket\\Engine\\Admin\\Beacon\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Admin/Beacon/ServiceProvider.php',
        'WP_Rocket\\Engine\\Admin\\Database\\Optimization' => __DIR__ . '/../..' . '/inc/Engine/Admin/Database/Optimization.php',
        'WP_Rocket\\Engine\\Admin\\Database\\OptimizationProcess' => __DIR__ . '/../..' . '/inc/Engine/Admin/Database/OptimizationProcess.php',
        'WP_Rocket\\Engine\\Admin\\Database\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Admin/Database/ServiceProvider.php',
        'WP_Rocket\\Engine\\Admin\\Database\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Admin/Database/Subscriber.php',
        'WP_Rocket\\Engine\\Admin\\Deactivation\\DeactivationIntent' => __DIR__ . '/../..' . '/inc/Engine/Admin/Deactivation/DeactivationIntent.php',
        'WP_Rocket\\Engine\\Admin\\Deactivation\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Admin/Deactivation/Subscriber.php',
        'WP_Rocket\\Engine\\Admin\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Admin/ServiceProvider.php',
        'WP_Rocket\\Engine\\Admin\\Settings\\Page' => __DIR__ . '/../..' . '/inc/Engine/Admin/Settings/Page.php',
        'WP_Rocket\\Engine\\Admin\\Settings\\Render' => __DIR__ . '/../..' . '/inc/Engine/Admin/Settings/Render.php',
        'WP_Rocket\\Engine\\Admin\\Settings\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Admin/Settings/ServiceProvider.php',
        'WP_Rocket\\Engine\\Admin\\Settings\\Settings' => __DIR__ . '/../..' . '/inc/Engine/Admin/Settings/Settings.php',
        'WP_Rocket\\Engine\\Admin\\Settings\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Admin/Settings/Subscriber.php',
        'WP_Rocket\\Engine\\CDN\\CDN' => __DIR__ . '/../..' . '/inc/Engine/CDN/CDN.php',
        'WP_Rocket\\Engine\\CDN\\RocketCDN\\APIClient' => __DIR__ . '/../..' . '/inc/Engine/CDN/RocketCDN/APIClient.php',
        'WP_Rocket\\Engine\\CDN\\RocketCDN\\AdminPageSubscriber' => __DIR__ . '/../..' . '/inc/Engine/CDN/RocketCDN/AdminPageSubscriber.php',
        'WP_Rocket\\Engine\\CDN\\RocketCDN\\CDNOptionsManager' => __DIR__ . '/../..' . '/inc/Engine/CDN/RocketCDN/CDNOptionsManager.php',
        'WP_Rocket\\Engine\\CDN\\RocketCDN\\DataManagerSubscriber' => __DIR__ . '/../..' . '/inc/Engine/CDN/RocketCDN/DataManagerSubscriber.php',
        'WP_Rocket\\Engine\\CDN\\RocketCDN\\NoticesSubscriber' => __DIR__ . '/../..' . '/inc/Engine/CDN/RocketCDN/NoticesSubscriber.php',
        'WP_Rocket\\Engine\\CDN\\RocketCDN\\RESTSubscriber' => __DIR__ . '/../..' . '/inc/Engine/CDN/RocketCDN/RESTSubscriber.php',
        'WP_Rocket\\Engine\\CDN\\RocketCDN\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/CDN/RocketCDN/ServiceProvider.php',
        'WP_Rocket\\Engine\\CDN\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/CDN/ServiceProvider.php',
        'WP_Rocket\\Engine\\CDN\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/CDN/Subscriber.php',
        'WP_Rocket\\Engine\\Cache\\AdminSubscriber' => __DIR__ . '/../..' . '/inc/Engine/Cache/AdminSubscriber.php',
        'WP_Rocket\\Engine\\Cache\\AdvancedCache' => __DIR__ . '/../..' . '/inc/Engine/Cache/AdvancedCache.php',
        'WP_Rocket\\Engine\\Cache\\Config\\ConfigSubscriber' => __DIR__ . '/../..' . '/inc/Engine/Cache/Config/ConfigSubscriber.php',
        'WP_Rocket\\Engine\\Cache\\Purge' => __DIR__ . '/../..' . '/inc/Engine/Cache/Purge.php',
        'WP_Rocket\\Engine\\Cache\\PurgeActionsSubscriber' => __DIR__ . '/../..' . '/inc/Engine/Cache/PurgeActionsSubscriber.php',
        'WP_Rocket\\Engine\\Cache\\PurgeExpired\\PurgeExpiredCache' => __DIR__ . '/../..' . '/inc/Engine/Cache/PurgeExpired/PurgeExpiredCache.php',
        'WP_Rocket\\Engine\\Cache\\PurgeExpired\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Cache/PurgeExpired/Subscriber.php',
        'WP_Rocket\\Engine\\Cache\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Cache/ServiceProvider.php',
        'WP_Rocket\\Engine\\Cache\\WPCache' => __DIR__ . '/../..' . '/inc/Engine/Cache/WPCache.php',
        'WP_Rocket\\Engine\\Capabilities\\Manager' => __DIR__ . '/../..' . '/inc/Engine/Capabilities/Manager.php',
        'WP_Rocket\\Engine\\Capabilities\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Capabilities/ServiceProvider.php',
        'WP_Rocket\\Engine\\Capabilities\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Capabilities/Subscriber.php',
        'WP_Rocket\\Engine\\Common\\Queue\\AbstractASQueue' => __DIR__ . '/../..' . '/inc/Engine/Common/Queue/AbstractASQueue.php',
        'WP_Rocket\\Engine\\Common\\Queue\\Cleaner' => __DIR__ . '/../..' . '/inc/Engine/Common/Queue/Cleaner.php',
        'WP_Rocket\\Engine\\Common\\Queue\\QueueInterface' => __DIR__ . '/../..' . '/inc/Engine/Common/Queue/QueueInterface.php',
        'WP_Rocket\\Engine\\Common\\Queue\\RUCSSQueueRunner' => __DIR__ . '/../..' . '/inc/Engine/Common/Queue/RUCSSQueueRunner.php',
        'WP_Rocket\\Engine\\CriticalPath\\APIClient' => __DIR__ . '/../..' . '/inc/Engine/CriticalPath/APIClient.php',
        'WP_Rocket\\Engine\\CriticalPath\\Admin\\Admin' => __DIR__ . '/../..' . '/inc/Engine/CriticalPath/Admin/Admin.php',
        'WP_Rocket\\Engine\\CriticalPath\\Admin\\Post' => __DIR__ . '/../..' . '/inc/Engine/CriticalPath/Admin/Post.php',
        'WP_Rocket\\Engine\\CriticalPath\\Admin\\Settings' => __DIR__ . '/../..' . '/inc/Engine/CriticalPath/Admin/Settings.php',
        'WP_Rocket\\Engine\\CriticalPath\\Admin\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/CriticalPath/Admin/Subscriber.php',
        'WP_Rocket\\Engine\\CriticalPath\\CriticalCSS' => __DIR__ . '/../..' . '/inc/Engine/CriticalPath/CriticalCSS.php',
        'WP_Rocket\\Engine\\CriticalPath\\CriticalCSSGeneration' => __DIR__ . '/../..' . '/inc/Engine/CriticalPath/CriticalCSSGeneration.php',
        'WP_Rocket\\Engine\\CriticalPath\\CriticalCSSSubscriber' => __DIR__ . '/../..' . '/inc/Engine/CriticalPath/CriticalCSSSubscriber.php',
        'WP_Rocket\\Engine\\CriticalPath\\DataManager' => __DIR__ . '/../..' . '/inc/Engine/CriticalPath/DataManager.php',
        'WP_Rocket\\Engine\\CriticalPath\\ProcessorService' => __DIR__ . '/../..' . '/inc/Engine/CriticalPath/ProcessorService.php',
        'WP_Rocket\\Engine\\CriticalPath\\RESTCSSSubscriber' => __DIR__ . '/../..' . '/inc/Engine/CriticalPath/RESTCSSSubscriber.php',
        'WP_Rocket\\Engine\\CriticalPath\\RESTWP' => __DIR__ . '/../..' . '/inc/Engine/CriticalPath/RESTWP.php',
        'WP_Rocket\\Engine\\CriticalPath\\RESTWPInterface' => __DIR__ . '/../..' . '/inc/Engine/CriticalPath/RESTWPInterface.php',
        'WP_Rocket\\Engine\\CriticalPath\\RESTWPPost' => __DIR__ . '/../..' . '/inc/Engine/CriticalPath/RESTWPPost.php',
        'WP_Rocket\\Engine\\CriticalPath\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/CriticalPath/ServiceProvider.php',
        'WP_Rocket\\Engine\\CriticalPath\\TransientTrait' => __DIR__ . '/../..' . '/inc/Engine/CriticalPath/TransientTrait.php',
        'WP_Rocket\\Engine\\Deactivation\\Deactivation' => __DIR__ . '/../..' . '/inc/Engine/Deactivation/Deactivation.php',
        'WP_Rocket\\Engine\\Deactivation\\DeactivationInterface' => __DIR__ . '/../..' . '/inc/Engine/Deactivation/DeactivationInterface.php',
        'WP_Rocket\\Engine\\Deactivation\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Deactivation/ServiceProvider.php',
        'WP_Rocket\\Engine\\HealthCheck\\ActionSchedulerCheck' => __DIR__ . '/../..' . '/inc/Engine/HealthCheck/ActionSchedulerCheck.php',
        'WP_Rocket\\Engine\\HealthCheck\\HealthCheck' => __DIR__ . '/../..' . '/inc/Engine/HealthCheck/HealthCheck.php',
        'WP_Rocket\\Engine\\HealthCheck\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/HealthCheck/ServiceProvider.php',
        'WP_Rocket\\Engine\\Heartbeat\\HeartbeatSubscriber' => __DIR__ . '/../..' . '/inc/Engine/Heartbeat/HeartbeatSubscriber.php',
        'WP_Rocket\\Engine\\Heartbeat\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Heartbeat/ServiceProvider.php',
        'WP_Rocket\\Engine\\License\\API\\Pricing' => __DIR__ . '/../..' . '/inc/Engine/License/API/Pricing.php',
        'WP_Rocket\\Engine\\License\\API\\PricingClient' => __DIR__ . '/../..' . '/inc/Engine/License/API/PricingClient.php',
        'WP_Rocket\\Engine\\License\\API\\User' => __DIR__ . '/../..' . '/inc/Engine/License/API/User.php',
        'WP_Rocket\\Engine\\License\\API\\UserClient' => __DIR__ . '/../..' . '/inc/Engine/License/API/UserClient.php',
        'WP_Rocket\\Engine\\License\\Renewal' => __DIR__ . '/../..' . '/inc/Engine/License/Renewal.php',
        'WP_Rocket\\Engine\\License\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/License/ServiceProvider.php',
        'WP_Rocket\\Engine\\License\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/License/Subscriber.php',
        'WP_Rocket\\Engine\\License\\Upgrade' => __DIR__ . '/../..' . '/inc/Engine/License/Upgrade.php',
        'WP_Rocket\\Engine\\Media\\Emojis\\EmojisSubscriber' => __DIR__ . '/../..' . '/inc/Engine/Media/Emojis/EmojisSubscriber.php',
        'WP_Rocket\\Engine\\Media\\ImageDimensions\\AdminSubscriber' => __DIR__ . '/../..' . '/inc/Engine/Media/ImageDimensions/AdminSubscriber.php',
        'WP_Rocket\\Engine\\Media\\ImageDimensions\\ImageDimensions' => __DIR__ . '/../..' . '/inc/Engine/Media/ImageDimensions/ImageDimensions.php',
        'WP_Rocket\\Engine\\Media\\ImageDimensions\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Media/ImageDimensions/Subscriber.php',
        'WP_Rocket\\Engine\\Media\\Lazyload\\AdminSubscriber' => __DIR__ . '/../..' . '/inc/Engine/Media/Lazyload/AdminSubscriber.php',
        'WP_Rocket\\Engine\\Media\\Lazyload\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Media/Lazyload/Subscriber.php',
        'WP_Rocket\\Engine\\Media\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Media/ServiceProvider.php',
        'WP_Rocket\\Engine\\Optimization\\AbstractOptimization' => __DIR__ . '/../..' . '/inc/Engine/Optimization/AbstractOptimization.php',
        'WP_Rocket\\Engine\\Optimization\\AdminServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Optimization/AdminServiceProvider.php',
        'WP_Rocket\\Engine\\Optimization\\AssetsLocalCache' => __DIR__ . '/../..' . '/inc/Engine/Optimization/AssetsLocalCache.php',
        'WP_Rocket\\Engine\\Optimization\\Buffer\\Optimization' => __DIR__ . '/../..' . '/inc/Engine/Optimization/Buffer/Optimization.php',
        'WP_Rocket\\Engine\\Optimization\\Buffer\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/Buffer/Subscriber.php',
        'WP_Rocket\\Engine\\Optimization\\CSSTrait' => __DIR__ . '/../..' . '/inc/Engine/Optimization/CSSTrait.php',
        'WP_Rocket\\Engine\\Optimization\\CacheDynamicResource' => __DIR__ . '/../..' . '/inc/Engine/Optimization/CacheDynamicResource.php',
        'WP_Rocket\\Engine\\Optimization\\ContentTrait' => __DIR__ . '/../..' . '/inc/Engine/Optimization/ContentTrait.php',
        'WP_Rocket\\Engine\\Optimization\\DeferJS\\AdminSubscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/DeferJS/AdminSubscriber.php',
        'WP_Rocket\\Engine\\Optimization\\DeferJS\\DeferJS' => __DIR__ . '/../..' . '/inc/Engine/Optimization/DeferJS/DeferJS.php',
        'WP_Rocket\\Engine\\Optimization\\DeferJS\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Optimization/DeferJS/ServiceProvider.php',
        'WP_Rocket\\Engine\\Optimization\\DeferJS\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/DeferJS/Subscriber.php',
        'WP_Rocket\\Engine\\Optimization\\DelayJS\\Admin\\Settings' => __DIR__ . '/../..' . '/inc/Engine/Optimization/DelayJS/Admin/Settings.php',
        'WP_Rocket\\Engine\\Optimization\\DelayJS\\Admin\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/DelayJS/Admin/Subscriber.php',
        'WP_Rocket\\Engine\\Optimization\\DelayJS\\HTML' => __DIR__ . '/../..' . '/inc/Engine/Optimization/DelayJS/HTML.php',
        'WP_Rocket\\Engine\\Optimization\\DelayJS\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Optimization/DelayJS/ServiceProvider.php',
        'WP_Rocket\\Engine\\Optimization\\DelayJS\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/DelayJS/Subscriber.php',
        'WP_Rocket\\Engine\\Optimization\\DynamicLists\\APIClient' => __DIR__ . '/../..' . '/inc/Engine/Optimization/DynamicLists/APIClient.php',
        'WP_Rocket\\Engine\\Optimization\\DynamicLists\\DataManager' => __DIR__ . '/../..' . '/inc/Engine/Optimization/DynamicLists/DataManager.php',
        'WP_Rocket\\Engine\\Optimization\\DynamicLists\\DynamicLists' => __DIR__ . '/../..' . '/inc/Engine/Optimization/DynamicLists/DynamicLists.php',
        'WP_Rocket\\Engine\\Optimization\\DynamicLists\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Optimization/DynamicLists/ServiceProvider.php',
        'WP_Rocket\\Engine\\Optimization\\DynamicLists\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/DynamicLists/Subscriber.php',
        'WP_Rocket\\Engine\\Optimization\\GoogleFonts\\AbstractGFOptimization' => __DIR__ . '/../..' . '/inc/Engine/Optimization/GoogleFonts/AbstractGFOptimization.php',
        'WP_Rocket\\Engine\\Optimization\\GoogleFonts\\Admin\\Settings' => __DIR__ . '/../..' . '/inc/Engine/Optimization/GoogleFonts/Admin/Settings.php',
        'WP_Rocket\\Engine\\Optimization\\GoogleFonts\\Admin\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/GoogleFonts/Admin/Subscriber.php',
        'WP_Rocket\\Engine\\Optimization\\GoogleFonts\\Combine' => __DIR__ . '/../..' . '/inc/Engine/Optimization/GoogleFonts/Combine.php',
        'WP_Rocket\\Engine\\Optimization\\GoogleFonts\\CombineV2' => __DIR__ . '/../..' . '/inc/Engine/Optimization/GoogleFonts/CombineV2.php',
        'WP_Rocket\\Engine\\Optimization\\GoogleFonts\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/GoogleFonts/Subscriber.php',
        'WP_Rocket\\Engine\\Optimization\\IEConditionalSubscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/IEConditionalSubscriber.php',
        'WP_Rocket\\Engine\\Optimization\\Minify\\AbstractMinifySubscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/Minify/AbstractMinifySubscriber.php',
        'WP_Rocket\\Engine\\Optimization\\Minify\\AdminSubscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/Minify/AdminSubscriber.php',
        'WP_Rocket\\Engine\\Optimization\\Minify\\CSS\\AbstractCSSOptimization' => __DIR__ . '/../..' . '/inc/Engine/Optimization/Minify/CSS/AbstractCSSOptimization.php',
        'WP_Rocket\\Engine\\Optimization\\Minify\\CSS\\AdminSubscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/Minify/CSS/AdminSubscriber.php',
        'WP_Rocket\\Engine\\Optimization\\Minify\\CSS\\Combine' => __DIR__ . '/../..' . '/inc/Engine/Optimization/Minify/CSS/Combine.php',
        'WP_Rocket\\Engine\\Optimization\\Minify\\CSS\\Minify' => __DIR__ . '/../..' . '/inc/Engine/Optimization/Minify/CSS/Minify.php',
        'WP_Rocket\\Engine\\Optimization\\Minify\\CSS\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/Minify/CSS/Subscriber.php',
        'WP_Rocket\\Engine\\Optimization\\Minify\\JS\\AbstractJSOptimization' => __DIR__ . '/../..' . '/inc/Engine/Optimization/Minify/JS/AbstractJSOptimization.php',
        'WP_Rocket\\Engine\\Optimization\\Minify\\JS\\Combine' => __DIR__ . '/../..' . '/inc/Engine/Optimization/Minify/JS/Combine.php',
        'WP_Rocket\\Engine\\Optimization\\Minify\\JS\\Minify' => __DIR__ . '/../..' . '/inc/Engine/Optimization/Minify/JS/Minify.php',
        'WP_Rocket\\Engine\\Optimization\\Minify\\JS\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/Minify/JS/Subscriber.php',
        'WP_Rocket\\Engine\\Optimization\\Minify\\ProcessorInterface' => __DIR__ . '/../..' . '/inc/Engine/Optimization/Minify/ProcessorInterface.php',
        'WP_Rocket\\Engine\\Optimization\\QueryString\\Remove' => __DIR__ . '/../..' . '/inc/deprecated/Engine/Optimization/QueryString/Remove.php',
        'WP_Rocket\\Engine\\Optimization\\QueryString\\RemoveSubscriber' => __DIR__ . '/../..' . '/inc/deprecated/Engine/Optimization/QueryString/RemoveSubscriber.php',
        'WP_Rocket\\Engine\\Optimization\\RUCSS\\AbstractAPIClient' => __DIR__ . '/../..' . '/inc/Engine/Optimization/RUCSS/AbstractAPIClient.php',
        'WP_Rocket\\Engine\\Optimization\\RUCSS\\Admin\\Database' => __DIR__ . '/../..' . '/inc/Engine/Optimization/RUCSS/Admin/Database.php',
        'WP_Rocket\\Engine\\Optimization\\RUCSS\\Admin\\Settings' => __DIR__ . '/../..' . '/inc/Engine/Optimization/RUCSS/Admin/Settings.php',
        'WP_Rocket\\Engine\\Optimization\\RUCSS\\Admin\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/RUCSS/Admin/Subscriber.php',
        'WP_Rocket\\Engine\\Optimization\\RUCSS\\Controller\\Filesystem' => __DIR__ . '/../..' . '/inc/Engine/Optimization/RUCSS/Controller/Filesystem.php',
        'WP_Rocket\\Engine\\Optimization\\RUCSS\\Controller\\Queue' => __DIR__ . '/../..' . '/inc/Engine/Optimization/RUCSS/Controller/Queue.php',
        'WP_Rocket\\Engine\\Optimization\\RUCSS\\Controller\\UsedCSS' => __DIR__ . '/../..' . '/inc/Engine/Optimization/RUCSS/Controller/UsedCSS.php',
        'WP_Rocket\\Engine\\Optimization\\RUCSS\\Cron\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/RUCSS/Cron/Subscriber.php',
        'WP_Rocket\\Engine\\Optimization\\RUCSS\\Database\\Queries\\UsedCSS' => __DIR__ . '/../..' . '/inc/Engine/Optimization/RUCSS/Database/Queries/UsedCSS.php',
        'WP_Rocket\\Engine\\Optimization\\RUCSS\\Database\\Row\\UsedCSS' => __DIR__ . '/../..' . '/inc/Engine/Optimization/RUCSS/Database/Row/UsedCSS.php',
        'WP_Rocket\\Engine\\Optimization\\RUCSS\\Database\\Schemas\\UsedCSS' => __DIR__ . '/../..' . '/inc/Engine/Optimization/RUCSS/Database/Schemas/UsedCSS.php',
        'WP_Rocket\\Engine\\Optimization\\RUCSS\\Database\\Tables\\UsedCSS' => __DIR__ . '/../..' . '/inc/Engine/Optimization/RUCSS/Database/Tables/UsedCSS.php',
        'WP_Rocket\\Engine\\Optimization\\RUCSS\\Frontend\\APIClient' => __DIR__ . '/../..' . '/inc/Engine/Optimization/RUCSS/Frontend/APIClient.php',
        'WP_Rocket\\Engine\\Optimization\\RUCSS\\Frontend\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Optimization/RUCSS/Frontend/Subscriber.php',
        'WP_Rocket\\Engine\\Optimization\\RUCSS\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Optimization/RUCSS/ServiceProvider.php',
        'WP_Rocket\\Engine\\Optimization\\RegexTrait' => __DIR__ . '/../..' . '/inc/Engine/Optimization/RegexTrait.php',
        'WP_Rocket\\Engine\\Optimization\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Optimization/ServiceProvider.php',
        'WP_Rocket\\Engine\\Optimization\\UrlTrait' => __DIR__ . '/../..' . '/inc/Engine/Optimization/UrlTrait.php',
        'WP_Rocket\\Engine\\Plugin\\InformationSubscriber' => __DIR__ . '/../..' . '/inc/Engine/Plugin/InformationSubscriber.php',
        'WP_Rocket\\Engine\\Plugin\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Plugin/ServiceProvider.php',
        'WP_Rocket\\Engine\\Plugin\\UpdaterApiCommonSubscriber' => __DIR__ . '/../..' . '/inc/Engine/Plugin/UpdaterApiCommonSubscriber.php',
        'WP_Rocket\\Engine\\Plugin\\UpdaterApiTools' => __DIR__ . '/../..' . '/inc/Engine/Plugin/UpdaterApiTools.php',
        'WP_Rocket\\Engine\\Plugin\\UpdaterSubscriber' => __DIR__ . '/../..' . '/inc/Engine/Plugin/UpdaterSubscriber.php',
        'WP_Rocket\\Engine\\Preload\\Activation\\Activation' => __DIR__ . '/../..' . '/inc/Engine/Preload/Activation/Activation.php',
        'WP_Rocket\\Engine\\Preload\\Activation\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Preload/Activation/ServiceProvider.php',
        'WP_Rocket\\Engine\\Preload\\Admin\\Settings' => __DIR__ . '/../..' . '/inc/Engine/Preload/Admin/Settings.php',
        'WP_Rocket\\Engine\\Preload\\Admin\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Preload/Admin/Subscriber.php',
        'WP_Rocket\\Engine\\Preload\\Controller\\CheckExcludedTrait' => __DIR__ . '/../..' . '/inc/Engine/Preload/Controller/CheckExcludedTrait.php',
        'WP_Rocket\\Engine\\Preload\\Controller\\CheckFinished' => __DIR__ . '/../..' . '/inc/Engine/Preload/Controller/CheckFinished.php',
        'WP_Rocket\\Engine\\Preload\\Controller\\ClearCache' => __DIR__ . '/../..' . '/inc/Engine/Preload/Controller/ClearCache.php',
        'WP_Rocket\\Engine\\Preload\\Controller\\CrawlHomepage' => __DIR__ . '/../..' . '/inc/Engine/Preload/Controller/CrawlHomepage.php',
        'WP_Rocket\\Engine\\Preload\\Controller\\LoadInitialSitemap' => __DIR__ . '/../..' . '/inc/Engine/Preload/Controller/LoadInitialSitemap.php',
        'WP_Rocket\\Engine\\Preload\\Controller\\PreloadUrl' => __DIR__ . '/../..' . '/inc/Engine/Preload/Controller/PreloadUrl.php',
        'WP_Rocket\\Engine\\Preload\\Controller\\Queue' => __DIR__ . '/../..' . '/inc/Engine/Preload/Controller/Queue.php',
        'WP_Rocket\\Engine\\Preload\\Cron\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Preload/Cron/Subscriber.php',
        'WP_Rocket\\Engine\\Preload\\Database\\Queries\\Cache' => __DIR__ . '/../..' . '/inc/Engine/Preload/Database/Queries/Cache.php',
        'WP_Rocket\\Engine\\Preload\\Database\\Rows\\CacheRow' => __DIR__ . '/../..' . '/inc/Engine/Preload/Database/Rows/CacheRow.php',
        'WP_Rocket\\Engine\\Preload\\Database\\Schemas\\Cache' => __DIR__ . '/../..' . '/inc/Engine/Preload/Database/Schemas/Cache.php',
        'WP_Rocket\\Engine\\Preload\\Database\\Tables\\Cache' => __DIR__ . '/../..' . '/inc/Engine/Preload/Database/Tables/Cache.php',
        'WP_Rocket\\Engine\\Preload\\Fonts' => __DIR__ . '/../..' . '/inc/Engine/Preload/Fonts.php',
        'WP_Rocket\\Engine\\Preload\\Frontend\\FetchSitemap' => __DIR__ . '/../..' . '/inc/Engine/Preload/Frontend/FetchSitemap.php',
        'WP_Rocket\\Engine\\Preload\\Frontend\\SitemapParser' => __DIR__ . '/../..' . '/inc/Engine/Preload/Frontend/SitemapParser.php',
        'WP_Rocket\\Engine\\Preload\\Frontend\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Preload/Frontend/Subscriber.php',
        'WP_Rocket\\Engine\\Preload\\Links\\AdminSubscriber' => __DIR__ . '/../..' . '/inc/Engine/Preload/Links/AdminSubscriber.php',
        'WP_Rocket\\Engine\\Preload\\Links\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Preload/Links/ServiceProvider.php',
        'WP_Rocket\\Engine\\Preload\\Links\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Preload/Links/Subscriber.php',
        'WP_Rocket\\Engine\\Preload\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Preload/ServiceProvider.php',
        'WP_Rocket\\Engine\\Preload\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Preload/Subscriber.php',
        'WP_Rocket\\Engine\\Support\\Data' => __DIR__ . '/../..' . '/inc/Engine/Support/Data.php',
        'WP_Rocket\\Engine\\Support\\Rest' => __DIR__ . '/../..' . '/inc/Engine/Support/Rest.php',
        'WP_Rocket\\Engine\\Support\\ServiceProvider' => __DIR__ . '/../..' . '/inc/Engine/Support/ServiceProvider.php',
        'WP_Rocket\\Engine\\Support\\Subscriber' => __DIR__ . '/../..' . '/inc/Engine/Support/Subscriber.php',
        'WP_Rocket\\Event_Management\\Event_Manager' => __DIR__ . '/../..' . '/inc/classes/event-management/class-event-manager.php',
        'WP_Rocket\\Event_Management\\Event_Manager_Aware_Subscriber_Interface' => __DIR__ . '/../..' . '/inc/classes/event-management/event-manager-aware-subscriber-interface.php',
        'WP_Rocket\\Event_Management\\Subscriber_Interface' => __DIR__ . '/../..' . '/inc/classes/event-management/subscriber-interface.php',
        'WP_Rocket\\Interfaces\\Render_Interface' => __DIR__ . '/../..' . '/inc/classes/interfaces/class-render-interface.php',
        'WP_Rocket\\Logger\\HTML_Formatter' => __DIR__ . '/../..' . '/inc/classes/logger/class-html-formatter.php',
        'WP_Rocket\\Logger\\Logger' => __DIR__ . '/../..' . '/inc/classes/logger/class-logger.php',
        'WP_Rocket\\Logger\\Stream_Handler' => __DIR__ . '/../..' . '/inc/classes/logger/class-stream-handler.php',
        'WP_Rocket\\Plugin' => __DIR__ . '/../..' . '/inc/Plugin.php',
        'WP_Rocket\\ServiceProvider\\Common_Subscribers' => __DIR__ . '/../..' . '/inc/classes/ServiceProvider/class-common-subscribers.php',
        'WP_Rocket\\ServiceProvider\\Options' => __DIR__ . '/../..' . '/inc/classes/ServiceProvider/class-options.php',
        'WP_Rocket\\Subscriber\\Admin\\Settings\\Beacon_Subscriber' => __DIR__ . '/../..' . '/inc/deprecated/subscriber/admin/Settings/class-beacon-subscriber.php',
        'WP_Rocket\\Subscriber\\Media\\Webp_Subscriber' => __DIR__ . '/../..' . '/inc/classes/subscriber/Media/class-webp-subscriber.php',
        'WP_Rocket\\Subscriber\\Optimization\\Dequeue_JQuery_Migrate_Subscriber' => __DIR__ . '/../..' . '/inc/deprecated/subscriber/Optimization/class-dequeue-jquery-migrate-subscriber.php',
        'WP_Rocket\\Subscriber\\Optimization\\Minify_HTML_Subscriber' => __DIR__ . '/../..' . '/inc/deprecated/subscriber/admin/Optimization/class-minify-html-subscriber.php',
        'WP_Rocket\\Subscriber\\Third_Party\\Plugins\\Images\\Webp\\EWWW_Subscriber' => __DIR__ . '/../..' . '/inc/classes/subscriber/third-party/plugins/Images/Webp/class-ewww-subscriber.php',
        'WP_Rocket\\Subscriber\\Third_Party\\Plugins\\Images\\Webp\\Imagify_Subscriber' => __DIR__ . '/../..' . '/inc/classes/subscriber/third-party/plugins/Images/Webp/class-imagify-subscriber.php',
        'WP_Rocket\\Subscriber\\Third_Party\\Plugins\\Images\\Webp\\Optimus_Subscriber' => __DIR__ . '/../..' . '/inc/classes/subscriber/third-party/plugins/Images/Webp/class-optimus-subscriber.php',
        'WP_Rocket\\Subscriber\\Third_Party\\Plugins\\Images\\Webp\\ShortPixel_Subscriber' => __DIR__ . '/../..' . '/inc/classes/subscriber/third-party/plugins/Images/Webp/class-shortpixel-subscriber.php',
        'WP_Rocket\\Subscriber\\Third_Party\\Plugins\\Images\\Webp\\Webp_Common' => __DIR__ . '/../..' . '/inc/classes/subscriber/third-party/plugins/Images/Webp/trait-webp-common.php',
        'WP_Rocket\\Subscriber\\Third_Party\\Plugins\\Images\\Webp\\Webp_Interface' => __DIR__ . '/../..' . '/inc/classes/subscriber/third-party/plugins/Images/Webp/webp-interface.php',
        'WP_Rocket\\Subscriber\\Third_Party\\Plugins\\Mobile_Subscriber' => __DIR__ . '/../..' . '/inc/classes/subscriber/third-party/plugins/class-mobile-subscriber.php',
        'WP_Rocket\\Subscriber\\Third_Party\\Plugins\\NGG_Subscriber' => __DIR__ . '/../..' . '/inc/classes/subscriber/third-party/plugins/class-ngg-subscriber.php',
        'WP_Rocket\\Subscriber\\Third_Party\\Plugins\\SyntaxHighlighter_Subscriber' => __DIR__ . '/../..' . '/inc/classes/subscriber/third-party/plugins/class-syntaxhighlighter-subscriber.php',
        'WP_Rocket\\Subscriber\\Tools\\Detect_Missing_Tags_Subscriber' => __DIR__ . '/../..' . '/inc/classes/subscriber/Tools/class-detect-missing-tags-subscriber.php',
        'WP_Rocket\\ThirdParty\\Hostings\\AbstractNoCacheHost' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/AbstractNoCacheHost.php',
        'WP_Rocket\\ThirdParty\\Hostings\\Cloudways' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/Cloudways.php',
        'WP_Rocket\\ThirdParty\\Hostings\\Dreampress' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/Dreampress.php',
        'WP_Rocket\\ThirdParty\\Hostings\\Godaddy' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/Godaddy.php',
        'WP_Rocket\\ThirdParty\\Hostings\\HostResolver' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/HostResolver.php',
        'WP_Rocket\\ThirdParty\\Hostings\\HostSubscriberFactory' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/HostSubscriberFactory.php',
        'WP_Rocket\\ThirdParty\\Hostings\\Kinsta' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/Kinsta.php',
        'WP_Rocket\\ThirdParty\\Hostings\\LiteSpeed' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/LiteSpeed.php',
        'WP_Rocket\\ThirdParty\\Hostings\\O2Switch' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/O2Switch.php',
        'WP_Rocket\\ThirdParty\\Hostings\\OneCom' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/OneCom.php',
        'WP_Rocket\\ThirdParty\\Hostings\\Pressable' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/Pressable.php',
        'WP_Rocket\\ThirdParty\\Hostings\\Savvii' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/Savvii.php',
        'WP_Rocket\\ThirdParty\\Hostings\\ServiceProvider' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/ServiceProvider.php',
        'WP_Rocket\\ThirdParty\\Hostings\\SpinUpWP' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/SpinUpWP.php',
        'WP_Rocket\\ThirdParty\\Hostings\\WPEngine' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/WPEngine.php',
        'WP_Rocket\\ThirdParty\\Hostings\\WPXCloud' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/WPXCloud.php',
        'WP_Rocket\\ThirdParty\\Hostings\\WordPressCom' => __DIR__ . '/../..' . '/inc/ThirdParty/Hostings/WordPressCom.php',
        'WP_Rocket\\ThirdParty\\NullSubscriber' => __DIR__ . '/../..' . '/inc/ThirdParty/NullSubscriber.php',
        'WP_Rocket\\ThirdParty\\Plugins\\Ads\\Adthrive' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/Ads/Adthrive.php',
        'WP_Rocket\\ThirdParty\\Plugins\\CDN\\Cloudflare' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/CDN/Cloudflare.php',
        'WP_Rocket\\ThirdParty\\Plugins\\ConvertPlug' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/ConvertPlug.php',
        'WP_Rocket\\ThirdParty\\Plugins\\Ecommerce\\BigCommerce' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/Ecommerce/BigCommerce.php',
        'WP_Rocket\\ThirdParty\\Plugins\\Ecommerce\\WooCommerceSubscriber' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/Ecommerce/WooCommerceSubscriber.php',
        'WP_Rocket\\ThirdParty\\Plugins\\I18n\\WPML' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/I18n/WPML.php',
        'WP_Rocket\\ThirdParty\\Plugins\\InlineRelatedPosts' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/InlineRelatedPosts.php',
        'WP_Rocket\\ThirdParty\\Plugins\\Jetpack' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/Jetpack.php',
        'WP_Rocket\\ThirdParty\\Plugins\\ModPagespeed' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/ModPagespeed.php',
        'WP_Rocket\\ThirdParty\\Plugins\\Optimization\\AMP' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/Optimization/AMP.php',
        'WP_Rocket\\ThirdParty\\Plugins\\Optimization\\Autoptimize' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/Optimization/Autoptimize.php',
        'WP_Rocket\\ThirdParty\\Plugins\\Optimization\\Ezoic' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/Optimization/Ezoic.php',
        'WP_Rocket\\ThirdParty\\Plugins\\Optimization\\Hummingbird' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/Optimization/Hummingbird.php',
        'WP_Rocket\\ThirdParty\\Plugins\\Optimization\\RocketLazyLoad' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/Optimization/RocketLazyLoad.php',
        'WP_Rocket\\ThirdParty\\Plugins\\Optimization\\WPMeteor' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/Optimization/WPMeteor.php',
        'WP_Rocket\\ThirdParty\\Plugins\\PDFEmbedder' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/PDFEmbedder.php',
        'WP_Rocket\\ThirdParty\\Plugins\\PWA' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/PWA.php',
        'WP_Rocket\\ThirdParty\\Plugins\\PageBuilder\\BeaverBuilder' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/PageBuilder/BeaverBuilder.php',
        'WP_Rocket\\ThirdParty\\Plugins\\PageBuilder\\Elementor' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/PageBuilder/Elementor.php',
        'WP_Rocket\\ThirdParty\\Plugins\\RevolutionSlider' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/RevolutionSlider.php',
        'WP_Rocket\\ThirdParty\\Plugins\\SEO\\AllInOneSEOPack' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/SEO/AllInOneSEOPack.php',
        'WP_Rocket\\ThirdParty\\Plugins\\SEO\\RankMathSEO' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/SEO/RankMathSEO.php',
        'WP_Rocket\\ThirdParty\\Plugins\\SEO\\SEOPress' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/SEO/SEOPress.php',
        'WP_Rocket\\ThirdParty\\Plugins\\SEO\\TheSEOFramework' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/SEO/TheSEOFramework.php',
        'WP_Rocket\\ThirdParty\\Plugins\\SEO\\Yoast' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/SEO/Yoast.php',
        'WP_Rocket\\ThirdParty\\Plugins\\Security\\WordFenceCompatibility' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/Security/WordFenceCompatibility.php',
        'WP_Rocket\\ThirdParty\\Plugins\\SimpleCustomCss' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/SimpleCustomCss.php',
        'WP_Rocket\\ThirdParty\\Plugins\\Smush' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/Smush.php',
        'WP_Rocket\\ThirdParty\\Plugins\\TheEventsCalendar' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/TheEventsCalendar.php',
        'WP_Rocket\\ThirdParty\\Plugins\\ThirstyAffiliates' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/ThirstyAffiliates.php',
        'WP_Rocket\\ThirdParty\\Plugins\\UnlimitedElements' => __DIR__ . '/../..' . '/inc/ThirdParty/Plugins/UnlimitedElements.php',
        'WP_Rocket\\ThirdParty\\ReturnTypesTrait' => __DIR__ . '/../..' . '/inc/ThirdParty/ReturnTypesTrait.php',
        'WP_Rocket\\ThirdParty\\ServiceProvider' => __DIR__ . '/../..' . '/inc/ThirdParty/ServiceProvider.php',
        'WP_Rocket\\ThirdParty\\SubscriberFactoryInterface' => __DIR__ . '/../..' . '/inc/ThirdParty/SubscriberFactoryInterface.php',
        'WP_Rocket\\ThirdParty\\Themes\\Avada' => __DIR__ . '/../..' . '/inc/ThirdParty/Themes/Avada.php',
        'WP_Rocket\\ThirdParty\\Themes\\Bridge' => __DIR__ . '/../..' . '/inc/ThirdParty/Themes/Bridge.php',
        'WP_Rocket\\ThirdParty\\Themes\\Divi' => __DIR__ . '/../..' . '/inc/ThirdParty/Themes/Divi.php',
        'WP_Rocket\\ThirdParty\\Themes\\Flatsome' => __DIR__ . '/../..' . '/inc/ThirdParty/Themes/Flatsome.php',
        'WP_Rocket\\ThirdParty\\Themes\\Jevelin' => __DIR__ . '/../..' . '/inc/ThirdParty/Themes/Jevelin.php',
        'WP_Rocket\\ThirdParty\\Themes\\MinimalistBlogger' => __DIR__ . '/../..' . '/inc/ThirdParty/Themes/MinimalistBlogger.php',
        'WP_Rocket\\ThirdParty\\Themes\\Polygon' => __DIR__ . '/../..' . '/inc/ThirdParty/Themes/Polygon.php',
        'WP_Rocket\\ThirdParty\\Themes\\ServiceProvider' => __DIR__ . '/../..' . '/inc/ThirdParty/Themes/ServiceProvider.php',
        'WP_Rocket\\ThirdParty\\Themes\\ThirdpartyTheme' => __DIR__ . '/../..' . '/inc/ThirdParty/Themes/ThirdpartyTheme.php',
        'WP_Rocket\\ThirdParty\\Themes\\Uncode' => __DIR__ . '/../..' . '/inc/ThirdParty/Themes/Uncode.php',
        'WP_Rocket\\ThirdParty\\Themes\\Xstore' => __DIR__ . '/../..' . '/inc/ThirdParty/Themes/Xstore.php',
        'WP_Rocket\\Traits\\Config_Updater' => __DIR__ . '/../..' . '/inc/classes/traits/trait-config-updater.php',
        'WP_Rocket\\Traits\\Memoize' => __DIR__ . '/../..' . '/inc/classes/traits/trait-memoize.php',
        'WP_Rocket\\deprecated\\DeprecatedClassTrait' => __DIR__ . '/../..' . '/inc/deprecated/DeprecatedClassTrait.php',
        'WP_Rocket\\deprecated\\Engine\\Media\\Embeds\\EmbedsSubscriber' => __DIR__ . '/../..' . '/inc/deprecated/Engine/Media/Embeds/EmbedsSubscriber.php',
        'WP_Rocket_Mobile_Detect' => __DIR__ . '/../..' . '/inc/classes/dependencies/mobiledetect/mobiledetectlib/Mobile_Detect.php',
        'WP_Rocket_WP_Async_Request' => __DIR__ . '/../..' . '/inc/classes/dependencies/wp-media/background-processing/wp-async-request.php',
        'WP_Rocket_WP_Background_Process' => __DIR__ . '/../..' . '/inc/classes/dependencies/wp-media/background-processing/wp-background-process.php',
    );

    public static function getInitializer(ClassLoader $loader)
    {
        return \Closure::bind(function () use ($loader) {
            $loader->prefixLengthsPsr4 = ComposerStaticInita9a3a96e670bf58ac4dde9996f742c3d::$prefixLengthsPsr4;
            $loader->prefixDirsPsr4 = ComposerStaticInita9a3a96e670bf58ac4dde9996f742c3d::$prefixDirsPsr4;
            $loader->classMap = ComposerStaticInita9a3a96e670bf58ac4dde9996f742c3d::$classMap;

        }, null, ClassLoader::class);
    }
}
